#!/usr/bin/env python

from glob import glob
import packages
import visualize
from van.pydeb import bin_to_py

UBUNTU = 'ftp.litnet.lt'
DEBIAN = 'ftp.litnet.lt'
PPA = 'ppa.launchpad.net_schooltool-owners'

def get_apt_list(archive, distro, pocket='main'):
    variables = dict(archive=archive, distro=distro, pocket=pocket)
    pattern = '%(distro)s/var/lib/apt/lists/%(archive)s_*_dists_' \
              '%(distro)s_%(pocket)s_binary-*_Packages' % variables
    files = glob(pattern)
    assert len(files) == 1
    return files[0]

def load_package_lists(names=()):
    print 'Loading repository lists...'
    lists = {}

    lists['Ubuntu-Karmic-universe'] = packages.Debs(
        get_apt_list(UBUNTU, 'karmic', 'universe'), names)
    lists['Ubuntu-Karmic-main'] = packages.Debs(
        get_apt_list(UBUNTU, 'karmic', 'main'), names)
    lists['Ubuntu-Lucid-universe'] = packages.Debs(
        get_apt_list(UBUNTU, 'lucid', 'universe'), names)
    lists['Ubuntu-Lucid-main'] = packages.Debs(
        get_apt_list(UBUNTU, 'lucid', 'main'), names)
    lists['Debian-Squeeze'] = packages.Debs(
        get_apt_list(DEBIAN, 'squeeze', 'main'), names)
    lists['PPA-Karmic'] = packages.Debs(
        get_apt_list(PPA, 'karmic'))
    lists['PPA-Lucid'] = packages.Debs(
        get_apt_list(PPA, 'lucid'))

    print 'Done.'

    print 'Merging lists...'

    lists['Ubuntu-Karmic'] = packages.Debs()
    # Order matters here
    lists['Ubuntu-Karmic'].update(lists['Ubuntu-Karmic-main'])
    lists['Ubuntu-Karmic'].update(lists['Ubuntu-Karmic-universe'])

    lists['Karmic'] = packages.Debs()
    # Order matters here
    lists['Karmic'].update(lists['PPA-Karmic'])
    lists['Karmic'].update(lists['Ubuntu-Karmic'])

    lists['Ubuntu-Lucid'] = packages.Debs()
    # Order matters here
    lists['Ubuntu-Lucid'].update(lists['Ubuntu-Lucid-main'])
    lists['Ubuntu-Lucid'].update(lists['Ubuntu-Lucid-universe'])

    lists['Lucid'] = packages.Debs()
    # Order matters here
    lists['Lucid'].update(lists['PPA-Lucid'])
    lists['Lucid'].update(lists['Ubuntu-Lucid'])

    print 'Done.'
    return lists


def overview_eggs_against_debs(eggs, debs, sort_key=None):
    log = visualize.Legendator(
        legend={
            'REM': ('MISS',
                    'This package is missing from the repository.'),
            'SAME': ('', ''),
            'DOWN': ('UP',
                     'This package needs to be upgraded in the repository.'),
            'UP': ('WARN',
                   'We want a lower version than is in the repository.'),
        },
        ommit_prefixes=['NEW'], # skip unknown packages in rep.
        )

    eggs = packages.MappedPackages(eggs)
    debs = packages.MappedPackages(debs)
    visualize.print_diff(eggs, debs, log=log,
                         with_legend=True, by_prefix=True,
                         sort_key=sort_key)


def diff_two_repos(source, target, sort_key=None):

    log = visualize.Legendator(
        legend={
            'REM': ('---',
                    'Package not in target repository.'),
            'NEW': ('+++',
                    'Package not in source repository.'),
            'SAME': ('===', 'Packages are same.'),
            'DOWN': ('>>>',
                     'Source is newer than target.'),
            'UP': ('<<<',
                   'Newer version already in target.'),
        },
        ommit_prefixes=['SAME'], # skip uninteresting packages.
        )

    visualize.print_diff(source, target, log=log,
                         with_legend=True, by_prefix=True,
                         sort_key=sort_key)


if __name__ == '__main__':

    # THIS ENABLES 'ANIMATED' GAUGES THAT UGLIFY MY EMACS
    # but keeps my sanity on, when processing Debian sid / Ubuntu universe+main
    # visualize.CLUTTER_STDOUT = True

    names = open('package_list', 'r').read().splitlines()
    lists = load_package_lists(names)

    py_names = [bin_to_py(name) for name in names]
    eggs = packages.Eggs('versions.cfg', py_names)

    sort_key = packages.loose_namespace_sort_key(
        ['', 'zope.', 'zope.app', 'zc.', 'z3c', 'schooltool'])

    visualize.print_header(
        'Overview of versions.cfg against Karmic+PPA', level=2)
    overview_eggs_against_debs(eggs, lists['Karmic'], sort_key=sort_key)

    visualize.print_header(
        'Overview of versions.cfg against Lucid+PPA', level=2)
    overview_eggs_against_debs(eggs, lists['Lucid'], sort_key=sort_key)

    visualize.print_header('Diff Karmic+PPA vs. Lucid+PPA', level=2)
    diff_two_repos(lists['Karmic'], lists['Lucid'], sort_key=sort_key)

    visualize.print_header('Diff Debian Squeeze vs. Ubuntu Lucid', level=2)
    diff_two_repos(lists['Debian-Squeeze'], lists['Ubuntu-Lucid'], sort_key=sort_key)

