#!/usr/bin/python
import os
import sys
import shutil
from subprocess import call, Popen, STDOUT, PIPE

package_list = open('package_list', 'r').read().splitlines()

here = os.path.abspath(os.path.dirname(__file__))

JAUNTY_APT_CONF = os.path.join(here, 'jaunty', 'etc', 'apt', 'apt.conf')
KARMIC_APT_CONF = os.path.join(here, 'karmic', 'etc', 'apt', 'apt.conf')

VERBOSE = False

working_dir = os.path.join(here, sys.argv[1])
assert os.path.exists(working_dir)
build_dir = os.path.join(working_dir, 'build')
log_dir = os.path.join(working_dir, 'logs')
if not os.path.exists(build_dir):
    os.mkdir(build_dir)
if not os.path.exists(log_dir):
    os.mkdir(log_dir)

SOURCE_PKG = {'python-zodb': 'zodb'}

def get_source_dir(package, version):
    package = SOURCE_PKG.get(package, package)
    return '%s-%s' % (package, version.split('-')[0])


def silent_check_call(*args, **kw):
    assert 'stdout' not in kw
    assert 'stderr' not in kw
    kw['stdout'] = PIPE
    kw['stderr'] = STDOUT
    p = Popen(*args, **kw)
    stdout = p.communicate()[0]
    if p.returncode != 0:
        print stdout
        raise Exception("Call Failed %s" % args[0])

def check_call(*args, **kw):
    assert 0 == call(*args, **kw)

print 'I: UPDATING'
check_call(['apt-get', '-c', JAUNTY_APT_CONF, 'update'])
check_call(['apt-get', '-c', KARMIC_APT_CONF, 'update'])

def cmp_version(v1, v2):
    #print "D: Compare %r, %r" % (v1, v2)
    if 0 == call(['dpkg', '--compare-versions', v1, '>>', v2]):
        return 1
    if 0 == call(['dpkg', '--compare-versions', v1, '=', v2]):
        return 0
    if 0 == call(['dpkg', '--compare-versions', v1, '<<', v2]):
        return -1
    raise Exception('oops')

def get_versions(pkg, apt_conf):
    versions = []
    for line in Popen(["apt-cache", "-c", apt_conf, 'show', pkg], stdout=PIPE, stderr=PIPE).communicate()[0].splitlines():
        if line.startswith('Version: '):
            versions.append(line[9:])
    return versions

def get_max_version(pkg, apt_conf):
    versions = get_versions(pkg, apt_conf)
    if not versions:
        return None
    versions.sort(cmp=cmp_version)
    versions.reverse()
    return versions[0]


for package in package_list:
    print 'I: WORKING ON PACKAGE %s' % package
    jaunty_max = get_max_version(package, JAUNTY_APT_CONF)
    karmic_max = get_max_version(package, KARMIC_APT_CONF)

    if jaunty_max is None:
        print 'E: %s NOT IN SOURCE' % package
        continue

        src_max = jaunty_max
        SRC_APT_CONF = JAUNTY_APT_CONF

    if karmic_max is not None and cmp_version(karmic_max, src_max) >= 0:
        continue # dest is higher or equal to src

    success_log = os.path.join(log_dir, '%s-%s.success' % (package, src_max))
    failed_log = os.path.join(log_dir, '%s-%s.failed' % (package, src_max))
    if os.path.exists(success_log):
        print 'I: %s already built, skipping' % package
        continue
    os.chdir(build_dir)
    if os.path.exists(os.path.join(build_dir, get_source_dir(package, src_max))):
        shutil.rmtree(os.path.join(build_dir, get_source_dir(package, src_max)))
    silent_check_call(['apt-get', '-c', SRC_APT_CONF, 'source', package])
    os.chdir(os.path.join(build_dir, get_source_dir(package, src_max)))
    check_call(['dch', '-D', 'karmic', '-i', '-b', 'Rebuild for karmic.'])
    p = Popen(['dpkg-buildpackage', '-rfakeroot', '-S'], stdout=PIPE, stderr=STDOUT)
    stdout = p.communicate()[0]
    if p.returncode:
        print 'E: BUILD FAILED: %s' % package
        f = open(failed_log, 'w')
    else:
        print 'I: BUILD SUCCESS: %s' % package
        if os.path.exists(failed_log):
            os.remove(failed_log)
        f = open(success_log, 'w')
    f.write(stdout)
    f.close()

print "I: Making Packages"
os.chdir(working_dir)
p = Popen(['apt-ftparchive', 'packages', 'build'], stdout=PIPE)
stdout = p.communicate()[0]
f = open(os.path.join(build_dir, 'Packages'), 'w')
f.write(stdout)
f.close()
