import doctest
import unittest

from packages import DebDesc, EggDesc


class SimpleDeb(DebDesc):

    def __init__(self, package, version):
        self['Package'] = package
        self['Version'] = version


def doctest_compare_versions():
    """
    Create some debs:

        >>> deb0a1 = SimpleDeb('zope.one', '3.4.0~a1-1')
        >>> deb0 = SimpleDeb('zope.one', '3.4.0-1')
        >>> deb0ubuntu1 = SimpleDeb('zope.one', '3.4.0-1ubuntu1')

    And some eggs:

        >>> egg0a1 = EggDesc('zope.one', '3.4.0a1')
        >>> egg0 = EggDesc('zope.one', '3.4.0')
        >>> egg1 = EggDesc('zope.one', '3.4.1')

    Check deb comparison:

        >>> deb0.cmp_ver(deb0ubuntu1)
        -1
        >>> deb0.cmp_ver(deb0a1)
        1

    Check egg comparison:

        >>> egg0.cmp_ver(egg1)
        -1
        >>> egg0.cmp_ver(egg0a1)
        1

    Debian revision does not matter when comparing with eggs:

        >>> egg0.cmp_ver(deb0)
        0
        >>> egg0.cmp_ver(deb0ubuntu1)
        0
        >>> egg1.cmp_ver(deb0)
        1

    The same result if we compare debs to eggs:

        >>> deb0.cmp_ver(egg0)
        0
        >>> deb0ubuntu1.cmp_ver(egg0)
        0
        >>> deb0.cmp_ver(egg1)
        -1

    Alpha versions are correctly handled:

        >>> egg0a1.cmp_ver(deb0)
        -1
        >>> egg0a1.cmp_ver(deb0a1)
        0
        >>> egg0.cmp_ver(deb0a1)
        1

    The same result if we swap sides:

        >>> deb0.cmp_ver(egg0a1)
        1
        >>> deb0a1.cmp_ver(egg0a1)
        0
        >>> deb0a1.cmp_ver(egg0)
        -1

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    suite = doctest.DocTestSuite(optionflags=optionflags)
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
