#!/usr/bin/python
import os, sys
from optparse import OptionParser
from subprocess import call, Popen, STDOUT, PIPE
import apt_pkg # apt-get install python-apt

import packages

def silent_check_call(*args, **kw):
    assert 'stdout' not in kw
    assert 'stderr' not in kw
    kw['stdout'] = PIPE
    kw['stderr'] = STDOUT
    p = Popen(*args, **kw)
    stdout = p.communicate()[0]
    if p.returncode != 0:
        print stdout
        raise Exception("Call Failed %s" % args[0])

def check_call(*args, **kw):
    assert 0 == call(*args, **kw)

def cmp_version(v1, v2):
    v1 = v1.split('-')[0]
    v2 = v2.split('-')[0]
    packages.require_apt()
    return apt_pkg.VersionCompare(v1, v2)

def get_versions(pkg, apt_conf):
    versions = []
    for line in Popen(["apt-cache", "-c", apt_conf, 'show', pkg], stdout=PIPE, stderr=PIPE).communicate()[0].splitlines():
        if line.startswith('Version: '):
            versions.append(line[9:])
    return versions

def get_max_version(pkg, apt_conf):
    versions = get_versions(pkg, apt_conf)
    if not versions:
        return None
    versions.sort(cmp=cmp_version)
    versions.reverse()
    return versions[0]

def get_apt_conf(distro):
    return os.path.join(distro, 'etc/apt/apt.conf')

def apt_get_update(distro):
    apt_conf = get_apt_conf(distro)
    if not os.path.isfile(apt_conf):
        sys.exit('File %s does not exist' % apt_conf)

    def mkdir_p(dirname):
        d = os.path.join(distro, dirname)
        if not os.path.isdir(d):
            os.makedirs(d)

    def touch(filename):
        dirname = os.path.dirname(filename)
        mkdir_p(dirname)
        f = open(os.path.join(distro, filename), 'w')
        f.close()

    mkdir_p('etc/apt/preferences.d')
    mkdir_p('var/cache/apt/archives/partial')
    touch('var/lib/apt/lists/lock')
    mkdir_p('var/lib/apt/lists/partial')
    touch('var/lib/dpkg/lock')
    touch('var/lib/dpkg/status')
    check_call(['apt-get', '-c', apt_conf, 'update'])


if __name__ == '__main__':

    parser = OptionParser()
    (options, args) = parser.parse_args()

    karmic = 'karmic'
    lucid = 'lucid'
    if len(args) == 2:
        karmic = args[0]
        lucid = args[1]

    KARMIC_APT_CONF = get_apt_conf(karmic)
    LUCID_APT_CONF = get_apt_conf(lucid)

    apt_get_update(karmic)
    apt_get_update(lucid)
    apt_get_update('squeeze')

